<?php

declare(strict_types=1);

namespace Drupal\es_attachment\EventSubscriber;

use Drupal\elasticsearch_connector\Event\QueryParamsEvent;
use Drupal\es_attachment\Helpers;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Actions during the indexing process of one element.
 */
class QueryEvent implements EventSubscriberInterface {

  /**
   * The Helpers service.
   *
   * @var \Drupal\es_attachment\Helpers
   */
  private Helpers $helpers;

  /**
   * Constructs a new QueryEvent object.
   *
   * @param \Drupal\es_attachment\Helpers $helpers
   *   The Helpers service.
   */
  public function __construct(Helpers $helpers) {
    $this->helpers = $helpers;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[QueryParamsEvent::class][] = ['alterQuery', 101];
    return $events;
  }

  /**
   * Add Attachment to search queries.
   *
   * @param \Drupal\elasticsearch_connector\Event\QueryParamsEvent $event
   *   The QueryParamsEvent object.
   */
  public function alterQuery(QueryParamsEvent $event): void {

    // Check if the index is using the attachment processor.
    $index = $this->helpers->loadIndex($event->getIndexName());
    $attachment_processor_enable = $this->helpers->processorEnable($index);

    // Only alter if processor is enabled.
    if (!$attachment_processor_enable) {
      return;
    }

    $params = $event->getParams();

    // Only alter if it's a search query.
    if (!isset($params['body']['query']['query_string'])) {
      return;
    }

    $fields = $this->helpers->getDocumentFields($index);
    // If no fields are found, return.
    if (count($fields) === 0) {
      return;
    }

    $originalBoolQuery = $params['body']['query'];
    if (isset($params['body']['query']['bool']['must'])) {
      $originalBoolQuery = $params['body']['query']['bool']['must'];
    }

    unset($params['body']['query']);

    $queryString = $originalBoolQuery['query_string']['query'];
    // Find a way to add boost.
    $boost = 1;

    // We need to change the bool query from must to should.
    // This is required to add support for nested and string queries.
    // Add min match param.
    $params['body']['query']['bool']['minimum_should_match'] = 1;
    $params['body']['query']['bool']['should'] = [];
    $params['body']['query']['bool']['should'][] = $originalBoolQuery;

    // Build nestedQuery.
    // @see https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-nested-query.html.
    foreach ($fields as $field_name) {
      $params['body']['query']['bool']['should'][] = [
        'nested' => [
          'path' => $field_name,
          'query' => [
            'bool' => [
              'must' => [
                'query_string' => [
                  "query" => $queryString,
                  'fields' => [
                    $field_name . '.attachment.content^' . $boost,
                  ],
                ],
              ],
            ],
          ],
        ],
      ];
    }

    // Replace query.
    $event->setParams($params);
  }

}
